//
// Copyright 2015-2021 by Garmin Ltd. or its subsidiaries.
// Subject to Garmin SDK License Agreement and Wearables
// Application Developer Agreement.
//

import Toybox.Communications;
import Toybox.Cryptography;
import Toybox.Lang;
import Toybox.Timer;

class AtHome
{

var usr;
var pwd;

private var api;

private var cb;

public function initialize()
{

    L.dbg_log(L.DBG_BASIC, "AtHome:initialize()");
    usr = C.settings["athome_usr"];
    pwd = C.settings["athome_pwd"];
    api = new Api(C.settings["athome_svr"], usr, pwd);
}

public function get_host()
{

    return api.get_host();
}

public function enumerate(cb)
{

    L.dbg_log(L.DBG_BASIC, "AtHome:enumerate()");

    if (api.get_host() == null) {
        cb.invoke(C.OUTLET_NONE, null); }

    C.set_alert("AtHome\nsearching...");

    me.cb = cb;

    var body = {
        "usr" => usr,
        "pwd" => pwd,
        "enumerate" => "wemo"
    };
    api.post("", body, method(:ah_enumerate));
}

public function ah_enumerate(code, data as Dictionary)
{
    var dev;

    //L.pr_json(L.DBG_HTTP, "athome_list", code, data);
    if (code != 200) {
        C.state = C.MAIN_ERROR;
        var msg = (code == Communications.BLE_CONNECTION_UNAVAILABLE) ? "No phone" :
                                                                        ("Error(" + code + ")");
        cb.invoke(C.OUTLET_ERROR,  msg + "\nTap to continue");
        return;
    }
    var devices = data["devices"] as Dictionary;
    for (var i = 0; i < devices.size(); i += 1) {
        dev = devices[i] as Dictionary;
        C.outlet_add(C.OUTLET_ATHOME,
                     dev["name"],
                     C.ICON_PLUG,
                     C.STATE_OFF,
                     method(:athome_ctrl),
                     dev["info"]);
    }
    cb.invoke(C.OUTLET_NONE, null);
}

public function athome_ctrl(cmd, outlet as Dictionary, state, cb)
{

//L.pr_json(L.DBG_BASIC, "athome_ctrl", 0, outlet);

    me.cb = cb;

    switch (cmd) {

    case C.CTRL_SCAN:
        athome_scan(outlet);
        return;

    case C.CTRL_GET:
        athome_get(outlet);
        return;

    case C.CTRL_SET:
        athome_onoff(state, outlet);
        return;

    }
    L.dbg_log(L.DBG_BASIC, "ctrl:" + "unknown command" + cmd);
    cb.invoke(false);
}

public function athome_scan(outlet as Dictionary)
{

    //L.dbg_log(L.DBG_BASIC, "AtHome:scan()");

    if (api.get_host() == null) {
        cb.invoke(C.OUTLET_NONE, null); }

    me.cb = cb;

    var info = outlet["info"] as Dictionary;

    var body = {
        "usr" => usr,
        "pwd" => pwd,
        "get" => "wemo",
        "addr" => info["addr"]
    };
    api.post("", body, method(:ah_scan));
}

public function ah_scan(code, data as Dictionary)
{
    var r;

    //L.pr_json(L.DBG_HTTP, "ah_scan", code, data);

    r = C.STATE_OFFLINE;
    switch (data["state"]) {

    case "on":
        r = C.STATE_ON;
        break;

    case "off":
        r = C.STATE_OFF;
        break;

    case "OFFLINE":
        r = C.STATE_OFFLINE;
        break;

    }
    cb.invoke(r);
}

public function athome_get(outlet as Dictionary)
{

    L.pr_json(L.DBG_BASIC, "athome_ctrl", 0, outlet);
cb.invoke(false);
}

public function athome_onoff(state, outlet as Dictionary)
{

    //L.pr_json(L.DBG_BASIC, "athome_onoff", 0, outlet);

    var info = outlet["info"] as Dictionary;

    var body = {
        "usr" => usr,
        "pwd" => pwd,
        "set" => "wemo",
        "state" => ((state == C.STATE_ON) ? "on" : "off"),
        "addr" => info["addr"]
    };
    api.post("", body, method(:ah_onoff));
}

public function ah_onoff(code, data as Dictionary)
{

    //L.pr_json(L.DBG_HTTP, "ah_scan", code, data);

    cb.invoke(true);
}

}
