//
// Copyright 2015-2021 by Garmin Ltd. or its subsidiaries.
// Subject to Garmin SDK License Agreement and Wearables
// Application Developer Agreement.
//

import Toybox.Communications;
import Toybox.Cryptography;
import Toybox.Lang;

class Tplink {

private var api;
private var token;
private var scan_url;

private var cb;

public function initialize()
{

    L.dbg_log(L.DBG_HTTP, "Tplink.initialize()");
    api = new Api(C.settings["tp_svr"], C.settings["tp_usr"], C.settings["tp_pwd"]);
}

public function get_host()
{

    return api.get_host();
}

public function enumerate(cb)
{

    L.dbg_log(L.DBG_BASIC, "Tplink:enumerate()");

    if (api.get_host() == null) {
        cb.invoke(C.OUTLET_WEMO, null); }

    C.set_alert("Tplink\nsearching...");

    me.cb = cb;

    var body = {
                    "method" => "login",
                    "params" => {
                                "appType" => "Kasa_Android",
                                "cloudUserName" => C.settings["tp_usr"],
                                "cloudPassword" => C.settings["tp_pwd"],
                                "terminalUUID" => C.fake_uuid()
                             }
               };
    api.post("", body, method(:tp_login));
}

public function tp_login(code, data as Dictionary)
{

    if (code != 200) {
        //L.dbg_log(L.DBG_HTTP, "tp_login failed:" + code);
L.pr_json(L.DBG_BASIC, "login failed", code, data);
        C.state = C.MAIN_ERROR;
        var msg = (code == Communications.BLE_CONNECTION_UNAVAILABLE) ? "No phone" :
                                                                        ("Error(" + code + ")");
        cb.invoke(C.OUTLET_ERROR,  msg + "\nTap to continue");
        return;
    }

    var json = data["result"] as Dictionary;
    token = json["token"] as Dictionary;
//L.dbg_log(L.DBG_HTTP, "token:" + json["token"]);
    var body = {
                    "method" => "getDeviceList"
               };
    api.post("?token=" + token, body, method(:tp_devices));
}

public function tp_add(name, type, state, url, id, parent)
{

//C.log("tp_add:" + name + ", " + url + ", " + id + ", " + parent);
    var icon = C.ICON_PLUG;
    if (type.find("Switch") != null) {
        icon = C.ICON_SWITCH; }
    else if (type.find("Strip") != null) {
        icon = C.ICON_STRIP; }

    C.outlet_add(C.OUTLET_TPLINK,
                 name,
                 icon,
                 state,
                 method(:tp_ctrl),
                 {
                    "type" => "tplink",
                    "url" => url,
                    "id" => id,
                    "parent" => parent
                 });
    return name;
}

private function children(info as Dictionary)
{
    var child;

    var childs = info["children"] as Array<Dictionary>;
    if (childs == null) {
        return; }

    var id = info["deviceId"];
    var str = "";
    for (var j = 0; j < childs.size(); j += 1) {
        child = childs[j];
        str += "," + tp_add(child["alias"], "Plug", child["state"], scan_url, child["id"], id);
    }
L.dbg_log(L.DBG_HTTP, "tplink children of " + info["alias"] + str);
}

//
//  Garmin has too many error codes (at least an error_code for the response
//  and an err_code for the data info that I know of).  Rather than try an
//  handle every error code just assume that the correct relay state is returned
//  and, if not, let the try/except handle the end cases
//
public function tp_got(code, data as Dictionary)
{
    var res;

//L.pr_json(L.DBG_HTTP, "tp_got:", code, data);
    try {
        var info = (((data["result"] as Dictionary)["responseData"] as Dictionary)["system"] as Dictionary)["get_sysinfo"] as Dictionary;
        res = (info["err_code"] == 0) ? info["relay_state"] : -1;
        if (info["err_code"] == 0) {
            res = (info["relay_state"] == 0) ? C.STATE_OFF : C.STATE_ON; }
        else {
            res = C.STATE_OFFLINE; }
        if (res == null) {
            res = C.STATE_OFFLINE; }
        children(info);
    } catch(e) {
        res = C.STATE_OFFLINE;
    }
    cb.invoke(res);
}

public function tp_scan(outlet as Dictionary, cb)
{
    var body;

    var info = outlet["info"] as Dictionary;
//L.dbg_log(L.DBG_HTTP, "tp_get:" + info["id"]);

    me.cb = cb;
    me.scan_url = info["url"];

    body = {
        "method" => "passthrough",
        "params" => {
            "deviceId" => info["id"],
            "requestData" => {
                "system" => {
                    "get_sysinfo" => null
                    }
                }
            }
        };
    api.set_host(info["url"]);
    api.post("?token=" + token, body, method(:tp_got));
}

public function tp_get(outlet as Dictionary, cb)
{

    L.dbg_log(L.DBG_HTTP, "tp_get:" + outlet["name"]);
}

public function tp_set(code, data as Dictionary)
{

//L.pr_json(L.DBG_HTTP, "tp_set:", code, data);
    cb.invoke(data["error_code"] == 0);
}

public function tp_onoff(state, outlet as Dictionary, cb)
{
    var body;

    var info = outlet["info"] as Dictionary;
//L.dbg_log(L.DBG_HTTP, outlet["name"] + "/" + state + ":" + info["id"] + info);

    me.cb = cb;

    var parent = info["parent"];
    if (parent != null) {
        body = {
            "method" => "passthrough",
            "params" => {
                "deviceId" => parent,
                "requestData" => {
                    "context" => {
                        "child_ids" => [ info["id"] ]
                    },
                    "system" => {
                        "set_relay_state" => {
                                "state" => state
                            }
                        }
                    }
                }
            };
    } else {
        body = {
            "method" => "passthrough",
            "params" => {
                "deviceId" => info["id"],
                "requestData" => {
                    "system" => {
                        "set_relay_state" => {
                                "state" => state
                            }
                        }
                    }
                }
            };
    }
    api.set_host(info["url"]);
    api.post("?token=" + token, body, method(:tp_set));
}

public function tp_ctrl(cmd, outlet as Dictionary, state, cb)
{

//L.pr_json(L.DBG_BASIC, "tp_ctrl", 0, outlet);
    switch (cmd) {

    case C.CTRL_SCAN:
        tp_scan(outlet, cb);
        return;

    case C.CTRL_GET:
        tp_get(outlet, cb);
        return;

    case C.CTRL_SET:
        tp_onoff(state, outlet, cb);
        return;

    }
    L.dbg_log(L.DBG_BASIC, "ctrl:" + "unknown command" + cmd);
}

public function tp_devices(code, data as Dictionary)
{
    var dev;

    var str = "";
    var json = data["result"] as Dictionary;
    var devices = json["deviceList"] as Array<Dictionary>;
    for (var i = 0; i < devices.size(); i += 1) {
        dev = devices[i] as Dictionary;
        str += "," + tp_add(dev["alias"], dev["deviceName"], C.STATE_OFFLINE, dev["appServerUrl"], dev["deviceId"], null);
    }
L.dbg_log(L.DBG_HTTP, "tplink device" + str);
    cb.invoke(C.OUTLET_WEMO, null);
}

}
